import { Column, Entity, ManyToOne, Unique } from 'typeorm';
import { AbstractEntity } from '../../abstract/entities/abstract.entity';
import { SpaceEntity } from '../../space/entities/space.entity';
import { AqiSpaceDailyPollutantStatsDto } from '../dtos';

@Entity({ name: 'space-daily-pollutant-stats' })
@Unique(['spaceUuid', 'eventDay', 'eventHour'])
export class AqiSpaceDailyPollutantStatsEntity extends AbstractEntity<AqiSpaceDailyPollutantStatsDto> {
  @Column({ nullable: false })
  public spaceUuid: string;

  @ManyToOne(() => SpaceEntity, (space) => space.presenceSensorDaily)
  space: SpaceEntity;

  @Column({ nullable: false })
  public eventDay: string;

  @Column({ nullable: false })
  public eventHour: number;

  @Column('float', { nullable: true })
  public pm1Min: number;

  @Column('float', { nullable: true })
  public pm1Avg: number;

  @Column('float', { nullable: true })
  public pm1Max: number;

  @Column('float', { nullable: true })
  public pm10Min: number;

  @Column('float', { nullable: true })
  public pm10Avg: number;

  @Column('float', { nullable: true })
  public pm10Max: number;

  @Column('float', { nullable: true })
  public pm25Min: number;

  @Column('float', { nullable: true })
  public pm25Avg: number;

  @Column('float', { nullable: true })
  public pm25Max: number;

  @Column('float', { nullable: true })
  public ch2oMin: number;

  @Column('float', { nullable: true })
  public ch2oAvg: number;

  @Column('float', { nullable: true })
  public ch2oMax: number;

  @Column('float', { nullable: true })
  public vocMin: number;

  @Column('float', { nullable: true })
  public vocAvg: number;

  @Column('float', { nullable: true })
  public vocMax: number;

  @Column('float', { nullable: true })
  public co2Min: number;

  @Column('float', { nullable: true })
  public co2Avg: number;

  @Column('float', { nullable: true })
  public co2Max: number;

  @Column('float', { nullable: true })
  public aqiMin: number;

  @Column('float', { nullable: true })
  public aqiAvg: number;

  @Column('float', { nullable: true })
  public aqiMax: number;

  constructor(partial: Partial<AqiSpaceDailyPollutantStatsEntity>) {
    super();
    Object.assign(this, partial);
  }
}
