import { Entity, Column, ManyToOne, ManyToMany, JoinTable } from 'typeorm';
import { SubspaceModelEntity } from './subspace-model.entity';
import { ProductEntity } from '@app/common/modules/product/entities/product.entity';
import { NewTagEntity } from '@app/common/modules/tag/entities/tag.entity';
import { SubspaceModelProductAllocationDto } from '../../dtos/subspace-model/subspace-model-product-allocation.dto';
import { AbstractEntity } from '@app/common/modules/abstract/entities/abstract.entity';

@Entity({ name: 'subspace_model_product_allocation' })
export class SubspaceModelProductAllocationEntity extends AbstractEntity<SubspaceModelProductAllocationDto> {
  @Column({
    type: 'uuid',
    default: () => 'gen_random_uuid()',
    nullable: false,
  })
  public uuid: string;

  @ManyToOne(
    () => SubspaceModelEntity,
    (subspaceModel) => subspaceModel.productAllocations,
    {
      nullable: false,
      onDelete: 'CASCADE',
    },
  )
  public subspaceModel: SubspaceModelEntity;

  @ManyToOne(() => ProductEntity, { nullable: false, onDelete: 'CASCADE' })
  public product: ProductEntity;

  @ManyToMany(() => NewTagEntity, (tag) => tag.subspaceModelAllocations, {
    cascade: true,
    onDelete: 'CASCADE',
  })
  @JoinTable({ name: 'subspace_model_product_tags' })
  public tags: NewTagEntity[];

  constructor(partial: Partial<SubspaceModelProductAllocationEntity>) {
    super();
    Object.assign(this, partial);
  }
}
