import { Column, Entity, JoinColumn, ManyToOne, OneToMany } from 'typeorm';
import { AbstractEntity } from '../../abstract/entities/abstract.entity';
import { TagModelDto } from '../dtos/tag-model.dto';
import { SpaceModelEntity } from './space-model.entity';
import { SubspaceModelEntity } from './subspace-model';
import { ProductEntity } from '../../product/entities';
import { TagEntity } from '../../space/entities/tag.entity';

@Entity({ name: 'tag_model' })
export class TagModel extends AbstractEntity<TagModelDto> {
  @Column({ type: 'varchar', length: 255 })
  tag: string;

  @ManyToOne(() => ProductEntity, (product) => product.tagModels, {
    nullable: false,
  })
  @JoinColumn({ name: 'product_id' })
  product: ProductEntity;

  @ManyToOne(() => SpaceModelEntity, (space) => space.tags, { nullable: true })
  @JoinColumn({ name: 'space_model_id' })
  spaceModel: SpaceModelEntity;

  @ManyToOne(() => SubspaceModelEntity, (subspace) => subspace.tags, {
    nullable: true,
  })
  @JoinColumn({ name: 'subspace_model_id' })
  subspaceModel: SubspaceModelEntity;

  @Column({
    nullable: false,
    default: false,
  })
  public disabled: boolean;

  @OneToMany(() => TagEntity, (tag) => tag.model)
  tags: TagEntity[];
}
