import { Entity, Column, ManyToOne, ManyToMany, JoinTable } from 'typeorm';
import { SpaceEntity } from './space.entity';
import { SpaceModelProductAllocationEntity } from '../../space-model/entities/space-model-product-allocation.entity';
import { ProductEntity } from '../../product/entities/product.entity';
import { NewTagEntity } from '../../tag/entities/tag.entity';
import { AbstractEntity } from '../../abstract/entities/abstract.entity';
import { SpaceProductAllocationDto } from '../dtos/space-product-allocation.dto';

@Entity({ name: 'space_product_allocation' })
export class SpaceProductAllocationEntity extends AbstractEntity<SpaceProductAllocationDto> {
  @Column({
    type: 'uuid',
    default: () => 'gen_random_uuid()',
    nullable: false,
  })
  public uuid: string;

  @ManyToOne(() => SpaceEntity, (space) => space.productAllocations, {
    nullable: false,
    onDelete: 'CASCADE',
  })
  public space: SpaceEntity;

  @ManyToOne(() => SpaceModelProductAllocationEntity, {
    nullable: true,
    onDelete: 'SET NULL',
  })
  public inheritedFromModel?: SpaceModelProductAllocationEntity;

  @ManyToOne(() => ProductEntity, { nullable: false, onDelete: 'CASCADE' })
  public product: ProductEntity;

  @ManyToMany(() => NewTagEntity)
  @JoinTable({ name: 'space_product_tags' })
  public tags: NewTagEntity[];

  constructor(partial: Partial<SpaceProductAllocationEntity>) {
    super();
    Object.assign(this, partial);
  }
}
