import { Entity, Column, ManyToOne, Unique, ManyToMany } from 'typeorm';
import { ProductEntity } from '../../product/entities';
import { ProjectEntity } from '../../project/entities';
import { AbstractEntity } from '../../abstract/entities/abstract.entity';
import { NewTagDto } from '../dtos/tag.dto';
import { SpaceModelProductAllocationEntity } from '../../space-model/entities/space-model-product-allocation.entity';
import { SubspaceModelProductAllocationEntity } from '../../space-model/entities/subspace-model/subspace-model-product-allocation.entity';
import { DeviceEntity } from '../../device/entities/device.entity';

@Entity({ name: 'new_tag' })
@Unique(['name', 'project'])
export class NewTagEntity extends AbstractEntity<NewTagDto> {
  @Column({
    type: 'uuid',
    default: () => 'gen_random_uuid()',
    nullable: false,
  })
  public uuid: string;

  @Column({
    type: 'varchar',
    length: 255,
    nullable: true,
  })
  name: string;

  @ManyToOne(() => ProductEntity, (product) => product.newTags, {
    nullable: true,
    onDelete: 'CASCADE',
  })
  public product: ProductEntity;

  @ManyToOne(() => ProjectEntity, (project) => project.tags, {
    nullable: false,
    onDelete: 'CASCADE',
  })
  public project: ProjectEntity;

  @ManyToMany(
    () => SpaceModelProductAllocationEntity,
    (allocation) => allocation.tags,
  )
  public spaceModelAllocations: SpaceModelProductAllocationEntity[];

  @ManyToMany(
    () => SubspaceModelProductAllocationEntity,
    (allocation) => allocation.tags,
  )
  public subspaceModelAllocations: SubspaceModelProductAllocationEntity[];

  @ManyToOne(() => DeviceEntity, (device) => device.tag)
  public devices: DeviceEntity[];

  constructor(partial: Partial<NewTagEntity>) {
    super();
    Object.assign(this, partial);
  }
}
