 WITH device_space AS (
  SELECT
    device.uuid AS device_id,
    device.created_at,
    device.space_device_uuid AS space_id,
    "device-status-log".event_time::date,
    "device-status-log".code,
    "device-status-log".value,
    "device-status-log".log
  FROM device
  LEFT JOIN "device-status-log"
    ON device.uuid = "device-status-log".device_id
  LEFT JOIN product
    ON product.uuid = device.product_device_uuid
  WHERE product.cat_name  = 'hjjcy'
),

average_pollutants as(
SELECT
event_time,
space_id,
AVG(CASE WHEN code = 'voc_value' THEN value::numeric END) AS voc_avg,
AVG(CASE WHEN code = 'pm1' THEN value::numeric END) AS pm1_avg,
AVG(CASE WHEN code = 'pm25_value' THEN value::numeric END) AS pm25_avg,
AVG(CASE WHEN code = 'pm10' THEN value::numeric END) AS pm10_avg,
AVG(CASE WHEN code = 'ch2o_value' THEN value::numeric END) AS ch2o_avg,
AVG(CASE WHEN code = 'co2_value' THEN value::numeric END) AS co2_avg,
AVG(CASE WHEN code = 'air_quality_index' then level_to_numeric(value) END) as air_quality_index

FROM device_space
--WHERE code IN ('pm25_value', 'pm10')
GROUP BY space_id, event_time
)

SELECT
  event_time::date as date,
  space_id,
  pm1_avg,
  pm25_avg,
  pm10_avg,
  voc_avg,
  ch2o_avg,
  co2_avg,
  --calculate_aqi('pm25', pm25_avg) AS aqi_pm25,
  --calculate_aqi('pm10', pm10_avg) AS aqi_pm10,
   GREATEST(
    calculate_aqi('pm25', pm25_avg),
    calculate_aqi('pm10', pm10_avg),
    calculate_aqi('voc_value', voc_avg),
    calculate_aqi('co2_value', co2_avg),
    calculate_aqi('ch2o_value', ch2o_avg)
  ) AS overall_AQI,
  air_quality_index as avg_space_device_aqi
FROM average_pollutants;